/*
 * Multi-TRRT.cpp
 *
 *  Created on: Nov 29, 2012
 *      Author: ddevaurs
 */

#include "API/Roadmap/graph.hpp"

#include "planner/Diffusion/Variants/Transition-RRT.hpp"
#include "planner/Diffusion/Variants/Multi-T-RRT.hpp"


/**
 * Initialize the Multi-T-RRT.
 * @return the number of nodes added to the graph
 */
unsigned MultiTRRT::init()
{
    // initialize the lists of waypoints and seeds
	waypoints.push_back(_q_start);
	waypointNodes.push_back(_Start);

    // add the additional user-defined configurations as waypoints and seeds (when relevant)
    for (unsigned i(0); i < (unsigned) _Robot->getRobotStruct()->nconf; ++i)
    {
        confPtr_t conf(new Configuration(_Robot, _Robot->getRobotStruct()->conf[i]->q));
        Node * node = new Node(_Graph, conf);
        _Graph->insertNode(node);
        ++m_nbInitNodes;

        waypoints.push_back(conf);
        waypointNodes.push_back(node);
    }

    // finalize the lists of waypoints and seeds
    waypoints.push_back(_q_goal);
    waypointNodes.push_back(_Goal);

    // create the T-RRT expansion
    expansion = new TransitionExpansion(_Graph, _Goal);

    // initialize the Tree Planner
    return TreePlanner::init();
}
