/*
 *  StompReplanner.cpp
 *  libmove3D-planners 
 *
 *  Created by Benjamin Vadant on 01/02/13.
 *  Based on Jim Mainprice replanningAlgorithms.hpp/cpp
 *  Copyright 2013 LAAS/CNRS. All rights reserved.
 *
 */

#include "StompReplanner.hpp"
#include "ReplanningSimulator.hpp"

#include "API/project.hpp"

#include "planner/planEnvironment.hpp"
#include "planner/plannerFunctions.hpp"
#include "planner/TrajectoryOptim/Stomp/stompOptimizer.hpp"
#include "planner/TrajectoryOptim/trajectoryOptim.hpp"
#include "planner/TrajectoryOptim/Classic/costOptimization.hpp"
#include "planner/cost_space.hpp"
#include "planner/Diffusion/Variants/Star-RRT.hpp"

#include "HRICS_costspace.hpp"

#include "LightPlanner-pkg.h"
#include "Graphic-pkg.h"
#include "Collision-pkg.h"

#include "move3d-headless.h"

#include <boost/thread/thread.hpp>
#include <sys/time.h>

using namespace std;
using namespace tr1;

StompReplanner::StompReplanner(Robot* r) : SimpleReplanner(r)
{
    
}

StompReplanner::~StompReplanner()
{
    
}

bool StompReplanner::init()
{
    return SimpleReplanner::init();
}

void StompReplanner::run()
{
    isPlanning = true;
    cout << "StompReplanner :: Start replanning step : " << __func__ << endl;
    
    if(!optimizer->humanHasMoved())
    {
        cout << "StompReplanner :: human has not moved => no replanning"<<endl;
        planningSucceeded = true;
    }
    else
    {
        planningSucceeded = false;

        PlanEnv->setBool(   PlanParam::trajStompWithTimeLimit, true );
        PlanEnv->setDouble( PlanParam::trajStompTimeLimit, timeLimitRePlanning-0.3 );// Set 0.3 seconds to do the concatenation of trajectories

        idRun++;

        API::Trajectory newPortion = currentTrajectory.extractSubTrajectory( positionSwitch, currentTrajectory.getRangeMax(), false);
        cout << "newPortion.getNbOfPaths() : " << newPortion.getNbOfPaths() << endl;
        cout << "newPortion.getRangeMax() : " << newPortion.getRangeMax() << endl;

        if( newPortion.getNbOfPaths() != 0 )
        {

            optimizer->setSource(newPortion.getBegin());                                        //STOMP OPtimizer
            optimizer->setTarget(newPortion.getEnd());                                        //STOMP OPtimizer

            double distN=newPortion.getCourbe()[0]->getBegin()->dist(*(newPortion.getCourbe().back()->getEnd()));
            double distC=currentTrajectory.getCourbe()[0]->getBegin()->dist(*(currentTrajectory.getCourbe().back()->getEnd()));


            PlanEnv->setInt(PlanParam::nb_pointsOnTraj,int((distN/distC)*100));
            PlanEnv->setDouble(PlanParam::trajOptimStdDev,(distN/distC)*0.000035);

            cout<<"ENVSTOMP "<<PlanEnv->getInt(PlanParam::nb_pointsOnTraj)<<";"<<PlanEnv->getDouble(PlanParam::trajOptimStdDev)<<endl;


            traj_optim_runStompNoInit( idRun, newPortion );                                   //trajectoryOptim


            // Get the new trajectory and store to draw
            API::Trajectory bestNewPortion = optimizer->getBestTraj();
            cout << "final_traj.getRangeMax() : " << bestNewPortion.getRangeMax() << endl;
            globalRePlanningEnv->storeTrajectoryToDraw( bestNewPortion, 0 );

            std::pair<bool,API::Trajectory> concatenatedTrajectory = concatenateToCurrentTrajectory( bestNewPortion );
            planningSucceeded = concatenatedTrajectory.first;

            if( planningSucceeded ) {
                currentTrajectory = concatenatedTrajectory.second;
            }
            else {
                cout << "Could not concatenate trajectory" << endl;
            }

        }
        else {
            cout << "newPortion.getNbOfPaths() : 0" << endl;
        }

        if( !planningSucceeded ) {
            cout << "StompReplanner :: Replanning failed" << endl;
        }
    }
    cout << "StompReplanner :: End replanning step : " << __func__ << endl;
    isPlanning = false;
}
